/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.cell.core.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.dom.XElement;

public class ProgressBarCell extends ResizeCell<Double> {

  public static interface ProgressBarAppearance {

    void render(SafeHtmlBuilder sb, Double value, int width);

    void updateText(XElement parent, String text);

  }

  protected final ProgressBarAppearance appearance;

  private int increment = 10;

  public ProgressBarCell() {
    this(GWT.<ProgressBarAppearance> create(ProgressBarAppearance.class));
  }

  public ProgressBarCell(ProgressBarAppearance appearance) {
    this.appearance = appearance;
  }

  /**
   * Returns the progress bar appearance.
   * 
   * @return the appearance
   */
  public ProgressBarAppearance getAppearance() {
    return appearance;
  }

  /**
   * Returns the bar's increment value.
   * 
   * @return the increment the increment
   */
  public int getIncrement() {
    return increment;
  }

  @Override
  public void render(Context context, Double value, SafeHtmlBuilder sb) {
    appearance.render(sb, value, getWidth());
  }

  /**
   * Resets the progress bar value to 0 and text to empty string.
   */
  public void reset(Context context, XElement parent) {
    updateProgress(context, parent, 0, "");
  }

  /**
   * The number of progress update segments to display within the progress bar
   * (defaults to 10). If the bar reaches the end and is still updating, it will
   * automatically wrap back to the beginning.
   * 
   * @param increment the new increment
   */
  public void setIncrement(int increment) {
    this.increment = increment;
  }

  /**
   * Updates the progress bar value, and optionally its text. If the text
   * argument is not specified, any existing text value will be unchanged. To
   * blank out existing text, pass "". Note that even if the progress bar value
   * exceeds 1, it will never automatically reset -- you are responsible for
   * determining when the progress is complete and calling {@link #reset} to
   * clear and/or hide the control.
   * 
   * @param value A value between 0 and 1 (e.g., .5, defaults to 0)
   * @param text The string to display in the progress text element or null.
   */
  public void updateProgress(Context context, XElement parent, double value, String text) {
    value = Math.min(Math.max(value, 0), 1);

    setValue(context, parent, value);

    if (text != null) {
      updateText(parent, text);
    }
  }

  /**
   * Updates the progress bar text. If specified, textEl will be updated,
   * otherwise the progress bar itself will display the updated text.
   * 
   * @param text The string to display in the progress text element
   */
  public void updateText(XElement parent, String text) {
    appearance.updateText(parent, text);
  }

}
