/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.cell.core.client.form;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.ValueUpdater;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.NativeEvent;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.widget.core.client.form.NumberPropertyEditor;

public class SpinnerFieldCell<N extends Number> extends NumberInputCell<N> {

  public interface SpinnerFieldAppearance extends TwinTriggerFieldAppearance {

  }

  private Number minValue = Double.MIN_VALUE;
  private Number maxValue = Double.MAX_VALUE;

  public SpinnerFieldCell(NumberPropertyEditor<N> propertyEditor) {
    this(propertyEditor, GWT.<SpinnerFieldAppearance> create(SpinnerFieldAppearance.class));
  }

  public SpinnerFieldCell(NumberPropertyEditor<N> propertyEditor, SpinnerFieldAppearance appearance) {
    super(propertyEditor, appearance);
  }

  /**
   * Returns the fields max value.
   * 
   * @param context the context
   * @return the max value
   */
  public Number getMaxValue(Context context) {
    return maxValue;
  }

  /**
   * Returns the field's minimum value.
   * 
   * @param context the context
   * @return the min value
   */
  public Number getMinValue(Context context) {
    return minValue;
  }

  /**
   * Sets the field's max allowable value.
   * 
   * @param maxValue the max value
   */
  public void setMaxValue(Number maxValue) {
    this.maxValue = maxValue.doubleValue();
  }

  /**
   * Sets the increment that should be used (defaults to 1d).
   * 
   * @param increment the increment to set.
   */
  public void setIncrement(N increment) {
    getPropertyEditor().setIncrement(increment);
  }

  /**
   * Sets the increment value.
   * 
   * @param context the context
   * @return the increment
   */
  public N getIncrement(Context context) {
    return getPropertyEditor().getIncrement();
  }

  /**
   * Sets the field's minimum allowed value.
   * 
   * @param minValue the minimum value
   */
  public void setMinValue(Number minValue) {
    this.minValue = minValue.doubleValue();
  }

  protected void doSpin(Cell.Context context, XElement parent, N value, ValueUpdater<N> updater, boolean up) {
    if (!isReadOnly()) {
      // use the current value in the input element
      String v = getInputElement(parent).getValue();

      if (!"".equals(v)) {
        try {
          value = getPropertyEditor().parse(v);
        } catch (Exception e) {
          e.printStackTrace();
        }
      }

      if (up) {
        N newVal = getPropertyEditor().incr(value);
        if (newVal.doubleValue() > maxValue.doubleValue() || newVal.doubleValue() < minValue.doubleValue()) {
          return;
        }
        updater.update(newVal);
        // clear the context as current value (before spin) will be used on
        // setValue
        clearViewData(context.getKey());
        setValue(context, parent, newVal);
      } else {
        N newVal = getPropertyEditor().decr(value);
        if (newVal.doubleValue() > maxValue.doubleValue() || newVal.doubleValue() < minValue.doubleValue()) {
          return;
        }
        updater.update(newVal);
        // clear the context as current value (before spin) will be used on
        // setValue
        clearViewData(context.getKey());
        setValue(context, parent, newVal);
      }
    }
  }

  @Override
  protected void onTriggerClick(Cell.Context context, XElement parent, NativeEvent event, N value,
      ValueUpdater<N> updater) {
    super.onTriggerClick(context, parent, event, value, updater);
    if (!isReadOnly() && !isDisabled()) {
      doSpin(context, parent, value, updater, true);
    }
  }

  @Override
  protected void onTwinTriggerClick(Cell.Context context, XElement parent, NativeEvent event, N value,
      ValueUpdater<N> updater) {
    super.onTwinTriggerClick(context, parent, event, value, updater);
    if (!isReadOnly() && !isDisabled()) {
      doSpin(context, parent, value, updater, false);
    }
  }

}
