/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.container;

import java.util.logging.Logger;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.HasOneWidget;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.GXTLogConfiguration;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.ContentPanel;
import com.sencha.gxt.widget.core.client.ContentPanel.ContentPanelAppearance;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent.BeforeExpandHandler;
import com.sencha.gxt.widget.core.client.event.ExpandEvent;
import com.sencha.gxt.widget.core.client.event.ExpandEvent.ExpandHandler;

public class AccordionLayoutContainer extends InsertResizeContainer implements HasOneWidget {

  public interface AccordionLayoutAppearance extends ContentPanelAppearance {

  }

  private class Handler implements BeforeExpandHandler, ExpandHandler {

    @Override
    public void onBeforeExpand(BeforeExpandEvent event) {
      AccordionLayoutContainer.this.onBeforeExpand(event);
    }

    @Override
    public void onExpand(ExpandEvent event) {
      AccordionLayoutContainer.this.onExpand(event);
    }

  }

  private ContentPanel activeWidget;
  private boolean activeOnTop;
  private boolean fill = true;
  private boolean hideCollapseTool;
  private boolean titleCollapse = true;

  private Handler handler = new Handler();
  
  private static Logger logger = Logger.getLogger(AccordionLayoutContainer.class.getName());

  public AccordionLayoutContainer() {
    setElement(DOM.createDiv());
  }
  
  /**
   * Returns true if the active item if first.
   * 
   * @return the active on top state
   */
  public boolean getActiveOnTop() {
    return activeOnTop;
  }
  
  @Override
  public Widget getWidget() {
    return activeWidget;
  }

  /**
   * Returns true if fill is enabled.
   * 
   * @return the fill state
   */
  public boolean isFill() {
    return fill;
  }

  /**
   * Returns true if the collapse tool is hidden.
   * 
   * @return the hide collapse tool state
   */
  public boolean isHideCollapseTool() {
    return hideCollapseTool;
  }

  /**
   * Returns true if title collapse is enabled.
   * 
   * @return the title collapse state
   */
  public boolean isTitleCollapse() {
    return titleCollapse;
  }

  /**
   * True to swap the position of each panel as it is expanded so that it
   * becomes the first item in the container, false to keep the panels in the
   * rendered order. (defaults to false).
   * 
   * @param activeOnTop true to keep the active item on top
   */
  public void setActiveOnTop(boolean activeOnTop) {
    this.activeOnTop = activeOnTop;
  }

  /**
   * True to adjust the active item's height to fill the available space in the
   * container, false to use the item's current height (defaults to true).
   * 
   * @param fill true to fill
   */
  public void setFill(boolean fill) {
    this.fill = fill;
  }

  /**
   * True to hide the contained panels' collapse/expand toggle buttons, false to
   * display them (defaults to false). When set to true, {@link #titleCollapse}
   * should be true also.
   * 
   * @param hideCollapseTool true to hide
   */
  public void setHideCollapseTool(boolean hideCollapseTool) {
    this.hideCollapseTool = hideCollapseTool;
  }

  /**
   * True to allow expand/collapse of each contained panel by clicking anywhere
   * on the title bar, false to allow expand/collapse only when the toggle tool
   * button is clicked (defaults to true). When set to false,
   * {@link #hideCollapseTool} should be false also.
   * 
   * @param titleCollapse true for title collapse
   */
  public void setTitleCollapse(boolean titleCollapse) {
    this.titleCollapse = titleCollapse;
  }

  @Override
  public void setWidget(IsWidget w) {
    setWidget(asWidgetOrNull(w));
  }

  @Override
  public void setWidget(Widget w) {
    if (activeWidget != w) {
      this.activeWidget = (ContentPanel)w;
      activeWidget.expand();
    }
  }

  @Override
  protected void doLayout() {
    if (activeWidget != null) {
      if (GXTLogConfiguration.loggingIsEnabled()) {
        logger.finest("doLayout");
      }
      Size size = getContainerTarget().getStyleSize();

      if (!fill) {
        activeWidget.setWidth(size.getWidth());
      } else {
        if (activeWidget.isExpanded()) {
          int hh = 0;
          for (int i = 0, len = getWidgetCount(); i < len; i++) {
            ContentPanel c = (ContentPanel) getWidget(i);

            if (c != activeWidget) {
              c.collapse();
              hh += (c.getHeader().getOffsetHeight());
              applyLayout(c, size.getWidth(), -1);
            }
          }
          activeWidget.setPixelSize(size.getWidth(), size.getHeight() - hh);
        } else {
          for (int i = 0, len = getWidgetCount(); i < len; i++) {
            ContentPanel c = (ContentPanel) getWidget(i);
            applyLayout(c, size.getWidth(), -1);
          }
        }
      }
    }
  }

  protected void onBeforeExpand(BeforeExpandEvent event) {
    if (activeWidget != null) {
      activeWidget.collapse();
    }

    activeWidget = (ContentPanel)event.getSource();

    if (activeOnTop) {
      getContainerTarget().insertChild(activeWidget.getElement(), 0);
    }
  }

  protected void onExpand(ExpandEvent event) {
    forceLayout();
  }

  @Override
  protected void onInsert(int index, Widget child) {
    super.onInsert(index, child);
    assert child instanceof ContentPanel : "AccordionLayoutContainer must be ContentPanels";

    ContentPanel cp = (ContentPanel) child;
    cp.addBeforeExpandHandler(handler);
    cp.addExpandHandler(handler);
    cp.setCollapsible(true);
    cp.setHideCollapseTool(hideCollapseTool);
    cp.setTitleCollapse(titleCollapse);
  }

  protected void onRemove(Widget child) {
    super.onRemove(child);

    ContentPanel cp = (ContentPanel) child;
    ComponentHelper.removeHandler(cp, BeforeExpandEvent.getType(), handler);
    ComponentHelper.removeHandler(cp, ExpandEvent.getType(), handler);
  };

}
