/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid.filters;

import java.util.ArrayList;
import java.util.List;

import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.util.Util;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.loader.FilterConfig;
import com.sencha.gxt.data.shared.loader.FilterConfigBean;
import com.sencha.gxt.widget.core.client.event.CheckChangeEvent;
import com.sencha.gxt.widget.core.client.menu.CheckMenuItem;

public class ListFilter<M, V> extends Filter<M, V> {

  private ListMenu<M, V> listMenu;
  private ListStore<V> store;
  
  @SuppressWarnings({"unchecked", "rawtypes"})
  public ListFilter(ValueProvider<M, V> valueProvider, ListStore<V> store) {
    super(valueProvider);
    
    this.store = store;
    menu = new ListMenu<M, V>(this, store);
    listMenu = (ListMenu) menu;
  }

  @SuppressWarnings("unchecked")
  @Override
  public List<FilterConfig> getFilterConfig() {
    FilterConfigBean config = new FilterConfigBean();
    config.setType("list");
    config.setValue(convertValueToString());
    return Util.createList(config);
  }
  
  /**
   * Returns the list store.
   * 
   * @return the list store
   */
  public ListStore<V> getStore() {
    return store;
  }

  @Override
  public Object getValue() {
    List<V> values = new ArrayList<V>();
    for (V m : listMenu.getSelected()) {
      values.add(m);
    }
    return values;
  }

  @SuppressWarnings("rawtypes")
  @Override
  public boolean isActivatable() {
    return getValue() != null && ((List) getValue()).size() > 0;
  }
  
  @SuppressWarnings("unchecked")
  protected String convertValueToString() {
    StringBuffer sb = new StringBuffer();
    List<V> temp = (List<V>)getValue();
    for (int i = 0; i < temp.size(); i++) {
      sb.append((i == 0 ? "" : "::") + temp.get(i));
    }
    return sb.toString();
  }
  
  @Override
  protected Class<V> getType() {
    return null;
  };

  protected void onCheckChange(CheckChangeEvent<CheckMenuItem> event) {
    setActive(isActivatable(), false);
    fireUpdate();
  }
  
  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  protected boolean validateModel(M model) {
    Object value = getValueProvider().getValue(model);
    List<Object> values = (List) getValue();
    return values.size() == 0 || values.contains(value);
  }

}
