/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid.filters;

import java.util.ArrayList;
import java.util.List;

import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.data.shared.loader.FilterConfig;
import com.sencha.gxt.data.shared.loader.NumberFilterHandler;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.form.NumberPropertyEditor;
import com.sencha.gxt.widget.core.client.grid.filters.RangeMenu.RangeItem;

public class NumericFilter<M, V extends Number> extends Filter<M, V> {

  public class DefaultNumericFilterMessages implements NumericFilterMessages {
    @Override
    public String emptyText() {
      return DefaultMessages.getMessages().numericFilter_emptyText();
    }
  }

  public interface NumericFilterMessages {
    String emptyText();
  }

  private final NumberPropertyEditor<V> propertyEditor;
  private List<RangeItem> rangeItems = new ArrayList<RangeItem>();
  private RangeMenu<M, V> rangeMenu;
  private NumericFilterMessages messages = new DefaultNumericFilterMessages();
  private int width = 125;

  @SuppressWarnings({"rawtypes", "unchecked"})
  public NumericFilter(ValueProvider<M, V> valueProvider, NumberPropertyEditor<V> propertyEditor) {
    super(valueProvider);
    
    this.propertyEditor = propertyEditor;

    setHandler(new NumberFilterHandler<V>(propertyEditor));

    rangeItems.add(RangeItem.LESSTHAN);
    rangeItems.add(RangeItem.GREATERTHAN);
    rangeItems.add(RangeItem.EQUAL);

    menu = new RangeMenu<M, V>(this);
    rangeMenu = (RangeMenu) menu;
    rangeMenu.setRangeItems(rangeItems);
    
    setWidth(getWidth());
  }
  
  protected boolean validateModel(M model) {
    Number val = getValueProvider().getValue(model);
    if (rangeMenu.eq != null && rangeMenu.eq.getCurrentValue() != null) {
      if (val == null || rangeMenu.eq.getCurrentValue().doubleValue() != val.doubleValue()) {
        return false;
      }
    }

    if (rangeMenu.lt != null && rangeMenu.lt.getCurrentValue() != null) {
      if (val == null || rangeMenu.lt.getCurrentValue().doubleValue() <= val.doubleValue()) {
        return false;
      }
    }

    if (rangeMenu.gt != null && rangeMenu.gt.getCurrentValue() != null) {
      if (val == null || rangeMenu.gt.getCurrentValue().doubleValue() >= val.doubleValue()) {
        return false;
      }
    }
    return true;
  };
  
  @Override
  public boolean isActivatable() {
    if (rangeMenu.eq != null && rangeMenu.eq.getCurrentValue() != null) {
      return true;
    }
    if (rangeMenu.lt != null && rangeMenu.lt.getCurrentValue() != null) {
      return true;
    }
    if (rangeMenu.gt != null && rangeMenu.gt.getCurrentValue() != null) {
      return true;
    }
    return false;
  }

  @SuppressWarnings("unchecked")
  @Override
  public List<FilterConfig> getFilterConfig() {
    return (List<FilterConfig>) getValue();
  }

  public NumericFilterMessages getMessages() {
    return messages;
  }

  @Override
  public Object getValue() {
    return rangeMenu.getValue();
  }

  public int getWidth() {
    return width;
  }

  public void setMessages(NumericFilterMessages messages) {
    this.messages = messages;
    rangeMenu.setEmptyText(messages.emptyText());
  }

  /**
   * Sets the field width (defaults to 125).
   * 
   * @param width the field width
   */
  public void setWidth(int width) {
    this.width = width;
    rangeMenu.setFieldWidth(width);
  }

  @Override
  protected Class<V> getType() {
    return null;
  }

  protected NumberPropertyEditor<V> getPropertyEditor() {
    return propertyEditor;
  }

}
